import sys
import os
from datetime import datetime

from optima.optima35 import OPTIMA35
from utils.utility import Utilities
from ui.main_window import Ui_MainWindow
from ui.exif_handler_window import ExifEditor

from PySide6 import QtWidgets
from PySide6.QtWidgets import (
    QMessageBox,
    QApplication,
    QMainWindow,
    QWidget,
    QVBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QCheckBox,
    QFileDialog,
    QHBoxLayout,
    QSpinBox,
)

class Optima35GUI(QMainWindow, Ui_MainWindow):
    def __init__(self, exif_file):
        super(Optima35GUI, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.o = OPTIMA35()
        self.u = Utilities()
        self.exif_file = exif_file
        self.exif_data = None

        self.setWindowTitle(f"{self.o.name} {self.o.version}")
        self.default_ui_layout()
        self.define_gui_interaction()

        if exif_file == "config/exif_example.yaml":
            self.change_statusbar("Using example exif...", 10000)

    def default_ui_layout(self):
        self.ui.png_quality_spinBox.setVisible(False)

    def define_gui_interaction(self):
        self.ui.input_folder_button.clicked.connect(self.browse_input_folder)
        self.ui.output_folder_button.clicked.connect(self.browse_output_folder)
        self.ui.start_button.clicked.connect(self.process)
        self.ui.image_type.currentIndexChanged.connect(self.update_quality_options)

        self.ui.exif_checkbox.stateChanged.connect(
            lambda state: self.handle_checkbox_state(state, 2, self.populate_exif)
        )
        self.ui.tabWidget.currentChanged.connect(self.on_tab_changed)
        self.ui.edit_exif_button.clicked.connect(self.open_exif_editor)
        self.ui.restart_button.clicked.connect(self.restart_app)

    def process(self):
        #self.ui.start_button.setEnabled(False)
        #self.ui.restart_button.setEnabled(False)
        self.check_options() # Get all user selected data
        input_folder_valid = os.path.exists(self.o.settings["input_folder"])
        output_folder_valid = os.path.exists(self.o.settings["output_folder"])
        if not input_folder_valid or not output_folder_valid:
            QMessageBox.warning(self, "Warning", f"Input location {input_folder_valid}\nOutput folder {output_folder_valid}...")
            return

        input_folder = self.o.settings["input_folder"]
        output_folder = self.o.settings["output_folder"]

        image_files = [
            f for f in os.listdir(input_folder) if f.lower().endswith((".png", ".jpg", ".jpeg", ".webp"))
        ]
        i = 1
        for image_file in image_files:
            input_path = os.path.join(input_folder, image_file)
            if self.o.settings["new_file_names"] != False:
                image_name = self.o.name_images(self.o.settings["new_file_names"], i, len(image_files), self.o.settings["invert_image_order"])
            else:
                image_name = os.path.splitext(image_file)[0]
            output_path = os.path.join(output_folder, image_name)

            self.o.process(input_path, output_path)
            self.handle_qprogressbar(i, len(image_files))
            i += 1

        QMessageBox.information(self, "Information", "Finished")
        #self.ui.start_button.setEnabled(True)
        #self.ui.restart_button.setEnabled(True)
        self.ui.progressBar.setValue(0)

    def open_exif_editor(self):
        """Open the EXIF Editor."""
        self.exif_editor = ExifEditor(self.exif_data)
        self.exif_editor.exif_data_updated.connect(self.update_exif_data)
        self.exif_editor.show()

    def update_exif_data(self, updated_exif_data):
        """Update the EXIF data."""
        self.exif_data = updated_exif_data
        self.populate_exif()

    def handle_checkbox_state(self, state, desired_state, action):
        """Perform an action based on the checkbox state and a desired state. Have to use lambda when calling."""
        if state == desired_state:
            action()

    def on_tab_changed(self, index):
        """Handle tab changes."""
        # chatgpt
        if index == 1:  # EXIF Tab
            self.handle_exif_file("read")
        elif index == 0:  # Main Tab
            self.handle_exif_file("write")

    def handle_exif_file(self, do):
        if do == "read":
            self.exif_data = self.u.read_yaml(self.exif_file)
        elif do == "write":
            self.u.write_yaml(self.exif_file, self.exif_data)

    def populate_exif(self):
        # partly chatGPT
        # Mapping of EXIF fields to comboboxes in the UI
        combo_mapping = {
            "make": self.ui.make_comboBox,
            "model": self.ui.model_comboBox,
            "lens": self.ui.lens_comboBox,
            "iso": self.ui.iso_comboBox,
            "image_description": self.ui.image_description_comboBox,
            "user_comment": self.ui.user_comment_comboBox,
            "artist": self.ui.artist_comboBox,
            "copyright_info": self.ui.copyright_info_comboBox,
        }
        self.populate_comboboxes(combo_mapping)

    def populate_comboboxes(self, combo_mapping):
        """Populate comboboxes with EXIF data."""
        # ChatGPT
        for field, comboBox in combo_mapping.items():
            comboBox.clear()  # Clear existing items
            comboBox.addItems(map(str, self.exif_data.get(field, [])))

    def update_quality_options(self):
            """Update visibility of quality settings based on selected format."""
            # ChatGPT
            selected_format = self.ui.image_type.currentText()
            # Hide all quality settings
            self.ui.png_quality_spinBox.setVisible(False)
            self.ui.jpg_quality_spinBox.setVisible(False)
            # Show relevant settings
            if selected_format == "jpg":
                self.ui.jpg_quality_spinBox.setVisible(True)
            elif selected_format == "webp":
                self.ui.jpg_quality_spinBox.setVisible(True)
            elif selected_format == "png":
                self.ui.png_quality_spinBox.setVisible(True)

    def browse_input_folder(self):
        folder = QFileDialog.getExistingDirectory(self, "Select Input Folder")
        if folder:
            self.ui.input_path.setText(folder)

    def browse_output_folder(self):
        folder = QFileDialog.getExistingDirectory(self, "Select Output Folder")
        if folder:
            self.ui.output_path.setText(folder)

    def change_statusbar(self, msg, timeout = 500):
        self.ui.statusBar.showMessage(msg, timeout)

    def handle_qprogressbar(self, current, total):
        progress = int((100 / total) * current)
        self.ui.progressBar.setValue(progress)

    def check_options(self):
        try:
            self.o.settings["input_folder"] = self.ui.input_path.text()
            self.o.settings["output_folder"] = self.ui.output_path.text()
            self.o.settings["file_format"] = self.ui.image_type.currentText()
            self.o.settings["jpg_quality"] = int(self.ui.jpg_quality_spinBox.text())
            self.o.settings["png_compression"] = int(self.ui.png_quality_spinBox.text())
            self.o.settings["invert_image_order"] = self.ui.revert_checkbox.isChecked()
            self.o.settings["grayscale"] = self.ui.grayscale_checkBox.isChecked()
            self.o.settings["copy_exif"] = self.ui.exif_copy_checkBox.isChecked()
            self.o.settings["own_exif"] = self.ui.exif_checkbox.isChecked()
            self.o.settings["font_size"] = self.ui.font_size_comboBox.currentIndex() + 1
            self.o.settings["optimize"] = self.ui.optimize_checkBox.isChecked()
            self.o.settings["own_date"] = self.ui.add_date_checkBox.isChecked()

            if self.ui.resize_checkbox.isChecked():
                self.o.settings["resize"] = int(self.ui.resize_spinBox.text())

            if self.ui.brightness_checkbox.isChecked():
                self.o.settings["brightness"] = int(self.ui.brightness_spinBox.text())

            if self.ui.contrast_checkbox.isChecked():
                self.o.settings["contrast"] = int(self.ui.contrast_spinBox.text())

            if self.ui.rename_checkbox.isChecked():
                if self.ui.filename.text() != "":
                    self.o.settings["new_file_names"] = self.ui.filename.text()
                else:
                    self.o.settings["new_file_names"] = False
            else:
                self.o.settings["new_file_names"] = False


            if self.ui.watermark_checkbox.isChecked():
                if self.ui.watermark_lineEdit.text() != "":
                    self.o.settings["watermark"] = self.ui.watermark_lineEdit.text()
                else:
                    self.o.settings["watermark"] = False
            else:
                self.o.settings["watermark"] = False

            if self.o.settings["own_exif"]:
                self.o.selected_exif = self.collect_selected_exif()
                if self.ui.add_date_checkBox.isChecked():
                    self.o.selected_exif["date_time_original"] = self.get_date()
                if self.ui.gps_checkBox.isChecked():
                    self.o.settings["gps"] = [self.ui.lat_lineEdit.text(), self.ui.long_lineEdit.text()]
                else:
                    self.o.settings["gps"] = False

        except Exception as e:
            print(f"Whoops: {e}")

    def get_date(self):
        date_input = self.ui.dateEdit.date().toString("yyyy-MM-dd")
        new_date = datetime.strptime(date_input, "%Y-%m-%d")
        return new_date.strftime("%Y:%m:%d 00:00:00")

    def collect_selected_exif(self):
        user_data = {}
        user_data["make"] = self.ui.make_comboBox.currentText()
        user_data["model"] = self.ui.model_comboBox.currentText()
        user_data["lens"] = self.ui.lens_comboBox.currentText()
        user_data["iso"] = self.ui.iso_comboBox.currentText()
        user_data["image_description"] = self.ui.image_description_comboBox.currentText()
        user_data["user_comment"] = self.ui.user_comment_comboBox.currentText()
        user_data["artist"] = self.ui.artist_comboBox.currentText()
        user_data["copyright_info"] = self.ui.copyright_info_comboBox.currentText()
        user_data["software"] = f"{self.o.name} {self.o.version}"
        return user_data

    def rebuild_ui(self):
        # Define the bash script to execute
        bash_script = "/home/sam/git/gitlab_public/optima-35/rebuild_ui.sh"
        os.system(bash_script)

    def restart_app(self):
        """Restarts the application."""
        self.rebuild_ui()
        # chatGPT
        python = sys.executable  # Path to the Python interpreter
        os.execv(python, [python] + sys.argv)

def main(exif_file):
    app = QtWidgets.QApplication(sys.argv)
    window = Optima35GUI(exif_file=exif_file)
    window.show()
    app.exec()

if __name__ == "__main__":
    if os.path.isfile("config/exif.yaml"):
        exif_file = "config/exif.yaml"
        print("Fall back to exif example file...")
    elif os.path.isfile("config/exif_example.yaml"):
        exif_file = "config/exif_example.yaml"
    else:
        print("Exif file missing, please ensure an exif file exist in config folder (exif.yaml, or exif_example_yaml)\nExiting...")
        exit()
    main(exif_file)
